import 'package:flutter/foundation.dart';
import 'package:flutter/widgets.dart';

class ContainerBuilder<T> extends InheritedWidget {

  ContainerBuilder({
    Key? key,
    required Widget child,
    required T Function() builder,
  })  : instance = builder.call(),
        super(child: child);

  final T instance;

  @override
  bool updateShouldNotify(ContainerBuilder<T> oldWidget) {
    return oldWidget.instance != instance;
  }

  static T of<T>(BuildContext context) {
    final result =
        context.dependOnInheritedWidgetOfExactType<ContainerBuilder<T>>();
    assert(result != null, "No ContainerBuilder with type $T found");
    return result!.instance;
  }
}
